<?php declare(strict_types=1);

namespace Drupal\commerce_product_feeds\Commands;

use Drupal\commerce_product_feeds\FeedGeneratorInterface;
use Drupal\commerce_product_feeds\Normalizer\Cache\NormalizedValueCacherInterface;
use Drupal\Core\File\FileSystemInterface;
use Drush\Commands\DrushCommands;

/**
 * Drush commands.
 */
class GenerationCommands extends DrushCommands {

  /**
   * Feed generator.
   *
   * @var \Drupal\commerce_product_feeds\FeedGeneratorInterface
   */
  protected $feedGenerator;

  /**
   * File System.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Cacher.
   *
   * @var \Drupal\commerce_product_feeds\Normalizer\Cache\NormalizedValueCacherInterface
   */
  protected $cacher;

  /**
   * {@inheritDoc}
   */
  public function __construct(FeedGeneratorInterface $feedGenerator, FileSystemInterface $fileSystem, NormalizedValueCacherInterface $cacher) {
    parent::__construct();
    $this->feedGenerator = $feedGenerator;
    $this->fileSystem = $fileSystem;
    $this->cacher = $cacher;
  }

  /**
   * The product-feed:generate command.
   *
   * This command generates a product feed in a specified format and outputs
   * the result to a file. The underlying logic is contained in the generator
   * service.
   *
   * @command product-feed:generate
   * @option string $filename Filename to output generated feed.
   * @option string $format Format for the feed; defaults to 'xml_atom'.
   */
  public function generate(string $filename, string $format = 'xml_atom') {
    $output = $this->feedGenerator->generate($format);
    $this->fileSystem
      ->saveData($output, $filename, FileSystemInterface::EXISTS_REPLACE);
  }

  /**
   * Alter cache types for clearing with drush cache:clear.
   *
   * @hook on-event cache-clear
   */
  public function alterCacheTypesToClear(array &$types) {
    $types['commerce-product-feeds'] = [$this->cacher, 'clearCache'];
  }

}
